searchData={"items":[{"type":"module","title":"CTE","doc":"The Closure Table for Elixir strategy, CTE for short,\nis a simple and elegant way of storing and working with\nhierarchies. It involves storing all paths through a tree,\nnot just those with a direct parent-child relationship. You\nmay want to chose this model, over the [Nested Sets model](https://en.wikipedia.org/wiki/Nested_set_model),\nshould you need referential integrity and to assign nodes to multiple trees.\n\nWith CTE you can navigate through hierarchies using a\nsimple [API](CTE.Adapter.html#functions), such as:\nfinding the ascendants and descendants of a node, inserting\nand deleting nodes, moving entire sub-trees or print them\nas a digraph (.dot) file.\n\nOptions available to most of the functions:\n\n- `:limit`, to limit the total number of nodes returned, when finding the ancestors or the descendants for nodes\n- `:itself`, accepting a boolean value. When `true`, the node used for finding its neighbors are returned as part of the results. Default: true\n- `:nodes`, accepting a boolean value. When `true`, the results are containing additional information about the nodes. Default: false","ref":"CTE.html"},{"type":"module","title":"Quick example. - CTE","doc":"In this example the: `:nodes` attribute, will be a Schema i.e. `Post`, `TreePath`, etc!\nIn our initial implementation, the nodes definitions must have at least the\n`:id`, as one of their properties. This caveat will be lifted\nin a later implementation.\n\n......\n\ntodo:  Update the docs\n\n\nPlease check the docs, the tests, and the examples folder, for more details.","ref":"CTE.html#module-quick-example"},{"type":"type","title":"CTE.config/0","doc":"","ref":"CTE.html#t:config/0"},{"type":"type","title":"CTE.name/0","doc":"","ref":"CTE.html#t:name/0"},{"type":"type","title":"CTE.nodes/0","doc":"","ref":"CTE.html#t:nodes/0"},{"type":"type","title":"CTE.paths/0","doc":"","ref":"CTE.html#t:paths/0"},{"type":"type","title":"CTE.repo/0","doc":"","ref":"CTE.html#t:repo/0"},{"type":"type","title":"CTE.t/0","doc":"","ref":"CTE.html#t:t/0"},{"type":"type","title":"CTE.table/0","doc":"","ref":"CTE.html#t:table/0"},{"type":"module","title":"CTE.Ecto","doc":"Most of the functions provided will accept the following options:\n\n- `:limit`, to limit the total number of nodes returned, when finding the ancestors or the descendants for nodes\n- `:itself`, accepting a boolean value. When `true`, the node used for finding its neighbors are returned as part of the results. Default: true\n- `:nodes`, accepting a boolean value. When `true`, the results are containing additional information about the nodes. Default: false","ref":"CTE.Ecto.html"},{"type":"function","title":"CTE.Ecto.ancestors/3","doc":"Retrieve the ancestors of a node","ref":"CTE.Ecto.html#ancestors/3"},{"type":"function","title":"CTE.Ecto.delete/3","doc":"Delete a leaf or a subtree.\n\nTo delete a leaf node set the limit option to: 1, and in this particular case\nall the nodes that reference the leaf will be assigned to the leaf's immediate ancestor\n\nIf limit is 0, then the leaf and its descendants will be deleted","ref":"CTE.Ecto.html#delete/3"},{"type":"function","title":"CTE.Ecto.descendants/3","doc":"Retrieve the descendants of a node","ref":"CTE.Ecto.html#descendants/3"},{"type":"function","title":"CTE.Ecto.insert/4","doc":"Insert a node under an existing ancestor","ref":"CTE.Ecto.html#insert/4"},{"type":"function","title":"CTE.Ecto.move/4","doc":"Move a subtree from one location to another.\n\nFirst, the subtree and its descendants are disconnected from its ancestors. And second, the subtree is inserted under the new parent (ancestor) and the subtree, including its descendants, is declared as descendants of all the new ancestors.","ref":"CTE.Ecto.html#move/4"},{"type":"function","title":"CTE.Ecto.tree/3","doc":"Calculate and return a \"tree\" structure containing the paths and the nodes under the given leaf/node","ref":"CTE.Ecto.html#tree/3"},{"type":"module","title":"CTE.Utils","doc":"Basic utilities for helping developing functionality about the `CTE` data structures","ref":"CTE.Utils.html"},{"type":"function","title":"CTE.Utils.print_dot/2","doc":"render a path hierarchy as a .dot string that you could use for drawing your results, using graphviz.  w⦿‿⦿t!\n\nUpon receiving something like this:\n\n    %{\n      nodes: %{\n        6 => %{\n          author: \"Rolie\",\n          comment: \"Everything is easier, than with the Nested Sets.\",\n          id: 6\n        },\n        8 => %{\n          author: \"Olie\",\n          comment: \"I’m sold! And I’ll use its Elixir implementation! <3\",\n          id: 8\n        },\n        9 => %{author: \"Polie\", comment: \"w⦿‿⦿t!\", id: 9}\n      },\n      paths: [\n               [6, 6, 0],\n               [6, 8, 1],\n               [8, 8, 0],\n               [6, 9, 1],\n               [9, 9, 0]\n             ]\n    }\n\nthis will output a .dot formatted string that you could use later for generating\nan image: `dot -Tpng  .dot -o  .png`","ref":"CTE.Utils.html#print_dot/2"},{"type":"function","title":"CTE.Utils.print_tree/3","doc":"print the tree at the console, using a custom function for selecting the info to be displayed\n\nThe print_tree/3 function receives the tree structure returned by the CTE, the id of an existing node we\nwant to start printing the tree with, followed by the options.","ref":"CTE.Utils.html#print_tree/3"},{"type":"function","title":"Options - CTE.Utils.print_tree/3","doc":"* `:callback` - a function that it is invoked for every node in the tree. Has two parameters:\n\n    * id - the id of the node we render, ao a node specific unique identifier i.e. node name, etc.\n    * nodes - the nodes received the tree structure, a map %{node_id => node}...any()\n\n    This function must return a tuple with two elements. First element is the name of the node we render,\n    the second one being any optional info you want to add.\n\n  * `:raw` - return a list of tuples if true. Each tuple will contain the depth of\n    the text returned from the callback. Useful for custom formatting the output of the print.\n\nExample, using the default options:\n\niex» {:ok, tree} = CTT.tree(1)\niex» CTE.Utils.print_tree(tree,1, callback: &({&2[&1].author <> \":\", &2[&1].comment}))\n\nOlie: Is Closure Table better than the Nested Sets?\n└── Rolie: It depends. Do you need referential integrity?\n └── Olie: Yeah.","ref":"CTE.Utils.html#print_tree/3-options"},{"type":"function","title":"CTE.Utils.tree_to_map/3","doc":"converts the given tree into a nested map\n\nThis function receives a tree structure returned by the CTE and\nthe id of an existing node we want to start from. It returns\na nested map representation of the tree.","ref":"CTE.Utils.html#tree_to_map/3"},{"type":"function","title":"Options - CTE.Utils.tree_to_map/3","doc":"* `:callback` - a function of arity one that accepts the current\n    node as input and outputs a formatted node. Defaults to `& &1`.\n\nExample:\n\niex» {:ok, tree} = CTT.tree(1)\niex» CTE.Utils.tree_to_map(tree, 6, callback: &Map.take(&1, [:text]))\n\n%{\n  \"children\" => [\n    %{\n      \"children\" => [],\n      \"id\" => 8,\n      \"node\" => %{text: \"I’m sold! And I’ll use its Elixir implementation! <3\"}\n    },\n    %{\"children\" => [], \"id\" => 9, \"node\" => %{text: \"w⦿‿⦿t!\"}}\n  ],\n  \"id\" => 6,\n  \"node\" => %{text: \"Everything is easier, than with the Nested Sets.\"}\n}","ref":"CTE.Utils.html#tree_to_map/3-options"},{"type":"extras","title":"Closure Table","doc":"# Closure Table\n\n[![Hex.pm](https://img.shields.io/hexpm/dt/closure_table.svg?maxAge=2592000)](https://hex.pm/packages/closure_table)\n[![Hexdocs.pm](https://img.shields.io/badge/api-hexdocs-brightgreen.svg)](https://hexdocs.pm/closure_table)\n\nThe Closure Table solution is a simple and elegant way of storing hierarchies. It involves storing all paths through a tree, not just those with a direct parent-child relationship. You may want to chose this model, over the [Nested Sets model](https://en.wikipedia.org/wiki/Nested_set_model), should you need referential integrity and to assign nodes to multiple trees.\n\nThroughout the various examples and tests, we will refer to the hierarchies depicted below, where we are modeling a hypothetical forum-like discussion between [Rolie, Olie and Polie](https://www.youtube.com/watch?v=LTkmaE_QWMQ), and their debate around the usefulness of this implementation :)\n\n![Closure Table](assets/closure_table.png)\n\nWarning:\n\n> This version introduces significant changes. We are removing the concept of a CT Adapter and focusing on using Ecto for the core functions. The (in-memory) adapter has been eliminated.","ref":"readme.html"},{"type":"extras","title":"Quick start - Closure Table","doc":"The current implementation is depending on Ecto ~> 3.1; using [Ecto.SubQuery](https://hexdocs.pm/ecto/Ecto.SubQuery.html)!\n\nFor this implementation to work you'll have to provide two tables, and the name of the Repo used by your application:\n\n1. a table containing the nodes, with `id` as the primary key. This is the default for now, but it will be configurable in the near future.\n2. a table name where the tree paths will be stored.\n3. the name of the Ecto.Repo, defined by your app\n\nIn a future version we will provide you with a convenient migration template to help you starting, but for now you must supply these tables.\n\nFor example, given you have the following Schemas for comments:\n\n```elixir\n      defmodule CT.Comment do\n        use Ecto.Schema\n        import Ecto.Changeset\n\n        @timestamps_opts [type: :utc_datetime]\n\n        schema \"comments\" do\n          field :text, :string\n          belongs_to :author, CT.Author\n\n          timestamps()\n        end\n      end\n```\n\nand a table used for storing the parent-child relationships\n\n```elixir\n\n      defmodule CT.TreePath do\n        use Ecto.Schema\n        import Ecto.Changeset\n        alias CT.Comment\n\n        @primary_key false\n\n        schema \"tree_paths\" do\n          belongs_to :parent_comment, Comment, foreign_key: :ancestor\n          belongs_to :comment, Comment, foreign_key: :descendant\n          field :depth, :integer, default: 0\n        end\n      end\n```\n\nwe can define the following module:\n\n```elixir\n\n      defmodule CT.MyCTE do\n        use CTE,\n        repo: CT.Repo,\n        nodes: CT.Comment,\n        paths: CT.TreePath\n      end\n```\n\nWe add our CTE Repo to the app's main supervision tree, like this:\n\n```elixir\n      defmodule CT.Application do\n        use Application\n\n        def start(_type, _args) do\n          children = [\n            CT.Repo,\n          ]\n\n          opts = [strategy: :one_for_one, name: CT.Supervisor]\n          Supervisor.start_link(children, opts)\n        end\n      end\n```\n\nrestart your application.\n\nThen using `iex -S mix`, we can start experimenting. Examples:\n\n```elixir\n      iex» CT.MyCTE.ancestors(9)\n      {:ok, [1, 4, 6]}\n\n      iex» {:ok, tree} = CT.MyCTE.tree(1)\n      {:ok,\n      %{\n      nodes: %{\n        6 => %CT.Comment{\n          __meta__: #Ecto.Schema.Metadata<:loaded, \"comments\">,\n          author: #Ecto.Association.NotLoaded ,\n          author_id: 2,\n          id: 6,\n          inserted_at: ~U[2019-07-21 01:10:35Z],\n          text: \"Everything is easier, than with the Nested Sets.\",\n          updated_at: ~U[2019-07-21 01:10:35Z]\n        },\n        8 => %CT.Comment{\n          __meta__: #Ecto.Schema.Metadata<:loaded, \"comments\">,\n          author: #Ecto.Association.NotLoaded ,\n          author_id: 1,\n          id: 8,\n          inserted_at: ~U[2019-07-21 01:10:35Z],\n          text: \"I’m sold! And I’ll use its Elixir implementation! <3\",\n          updated_at: ~U[2019-07-21 01:10:35Z]\n        },\n        ...\n\n      },\n      paths: [\n          [1, 1, 0],\n          [1, 2, 1],\n          [2, 2, 0],\n          [1, 3, 2],\n          [2, 3, 1],\n          [3, 3, 0],\n          [1, 7, 3],\n          [2, 7, 2],\n     ...\n            ]\n      }}\n```\n\nif you want to visualize a tree, you can do that too:\n\n```elixir\niex» CTE.Utils.print_tree(tree, 1, callback: &({&2[&1], &2[&1].text}))\n```\n\nand you may see this:\n\n```txt\nIs Closure Table better than the Nested Sets?\n├── It depends. Do you need referential integrity?\n│  └── Yeah\n│     └── Closure Table *has* referential integrity?\n└── Querying the data it's easier.\n   ├── What about inserting nodes?\n   └── Everything is easier, than with the Nested Sets.\n      ├── I'm sold! And I'll use its Elixir implementation! <3\n      └── w⦿‿⦿t!\n```\n\nPlease check the docs for more details and return from more updates!\n\nOh and there is a simple utility for helping you drawing your paths, using graphviz! From Rolie's comments, excerpt:\n\n![dot](assets/dot.dot.dot.png)\n\nMaybe useful?! If yes, then we'll let you find this function by yourself ;)\n\nhint: _check the tests <3_","ref":"readme.html#quick-start"},{"type":"extras","title":"Installation - Closure Table","doc":"If available on [Hex](https://hex.pm/packages/closure_table), you can install the package by adding `closure_table` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:closure_table, \"~> 2.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Contributing - Closure Table","doc":"- [Fork this project](https://github.com/florinpatrascu/closure_table/fork)\n- Create your feature branch (git checkout -b my-new-feature)\n- Setup database and test (`mix test`)\n- Commit your changes (`git commit -am 'Add some feature'`)\n- Push to the branch (`git push origin my-new-feature`)\n- Create new Pull Request","ref":"readme.html#contributing"},{"type":"extras","title":"License - Closure Table","doc":"```txt\nCopyright 2023 Florin T.PATRASCU & the Contributors\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\n  http://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.\n```","ref":"readme.html#license"},{"type":"extras","title":"CHANGELOG","doc":"# CHANGELOG","ref":"changelog.html"},{"type":"extras","title":"2.0.0 - CHANGELOG","doc":"This version is introducing major breaking changes. We drop the concept of a CT Adapter and focus on using Ecto, for the core functions. The (in)memory adapter is gone.\n\nAlso important: we're going \"process-less\", simple, streamlined, efficient and maybe a tad fast(er)\n\nThis is very much a work in progress, with a list of immediate todos as follow:\n\n- code cleanup and update the documentation\n- allow the user to define her own:\n  - primary key; name and maybe type\n  - foreign key; name and maybe type - optional\n  - callbacks (l8r)\n- telemetry and better logging\n- mix tasks for generating CT migrations\n- support for \"plugins\" ..","ref":"changelog.html#2-0-0"},{"type":"extras","title":"1.1.5 - CHANGELOG","doc":"- dependencies update","ref":"changelog.html#1-1-5"},{"type":"extras","title":"1.1.4 - CHANGELOG","doc":"- code cleanup","ref":"changelog.html#1-1-4"},{"type":"extras","title":"1.1.3 - CHANGELOG","doc":"- improved CT tree structure map representation. The node's children are a list now, to preserve their original order - suitable for rendering the tree in UI, etc.","ref":"changelog.html#1-1-3"},{"type":"extras","title":"1.1.2 - CHANGELOG","doc":"- convert a CT tree structure to a nested map representation of the tree. This can be used for exporting this data as json, and so on. Thanks @greg-rychlewski","ref":"changelog.html#1-1-2"},{"type":"extras","title":"1.1.1 - CHANGELOG","doc":"- requiring elixir 1.11 or newer\n- CTE.Supervisor code cleanup, accepting a name parameter now\n- fix the Memory Adapter example\n- module documentation update; example was old and was missing the depth\n- dependencies upgrade, for main and for the Ecto simple demo app","ref":"changelog.html#1-1-1"},{"type":"extras","title":"1.0.11 - CHANGELOG","doc":"Minor changes:\n\n- Dependencies updated to latest versions. New projects should no longer have dependency conflicts when installing closure_table\n- Refactored Memory adapter's `delete` function (credo recommendation)\n- Updated documentation to reflect that the options passed to the Ecto adapter's `move/3` are currently ignored\n- Updated tests to remove some warnings","ref":"changelog.html#1-0-11"},{"type":"extras","title":"1.0.10 - CHANGELOG","doc":"Major changes, and a couple of significant bug fixes.\n\n- stable version! w⦿‿⦿t!\n- deleting nodes using option: `limit: 1`, behave as expected now across adapters. To delete a leaf node set the limit option to: 1, and in this particular case all the nodes that reference the leaf will be assigned to the leaf's immediate ancestor. When using `limit: 0`, the leaf and all its descendants will be deleted!\n- added tests for ASCII printing for both adapter; memory and Ecto\n- the ancestors and the descendants are returned now using the proper order (based on their `:depth` value)\n- improved tests for those cases where the outcome is complex or confusing\n- updated dependencies\n\nWarning: please use the move function with care! I believe it will require some refactoring and you'll get an eternal place in the CT's Hall of Fame if you'd like contributing to improving it!","ref":"changelog.html#1-0-10"},{"type":"extras","title":"1.0.0 (rc) - CHANGELOG","doc":"This is a new version introducing breaking changes. Please continue to use 0.2.1 for production, until we hit 1.2 unless you know what you're doing =)\n\n- the CT Ecto demo setup is much friendlier now, and it is also seeding the new database with demo data\n- the labels support functions now\n- the paths returned will always have the depth, as the 3rd element (breaking chance)\n- support for pretty printing trees. Example:\n\n```elixir\n  {:ok, tree} = CTT.tree(1)\n  CTE.Utils.print_tree(tree,1, callback: &({&2[&1].author <> \":\", &2[&1].comment}))\n\n  Olie: Is Closure Table better than the Nested Sets?\n  └── Rolie: It depends. Do you need referential integrity?\n   └── Olie: Yeah.\n```","ref":"changelog.html#1-0-0-rc"},{"type":"extras","title":"0.2.1 - CHANGELOG","doc":"- support for immediate parent or child query #1; introducing the requirements for an extra field, in the tree_paths support table: `depth`","ref":"changelog.html#0-2-1"},{"type":"extras","title":"0.1.6 - CHANGELOG","doc":"- better tests covering the Ecto Adapter example","ref":"changelog.html#0-1-6"},{"type":"extras","title":"0.1.4 - CHANGELOG","doc":"- mix config fix.","ref":"changelog.html#0-1-4"},{"type":"extras","title":"0.1.3 - CHANGELOG","doc":"- fixing the examples using the CTE Adapter for Ecto - WIP","ref":"changelog.html#0-1-3"},{"type":"extras","title":"0.1.1 - CHANGELOG","doc":"- minor text adjustments\n- removing redundant content","ref":"changelog.html#0-1-1"},{"type":"extras","title":"0.1.0 - CHANGELOG","doc":"initial version","ref":"changelog.html#0-1-0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}